%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MATLAB code (miscellanea)
% Coded by: Eleftherios Giovanis 
% File: Matl_paper2_routine2.m
%
% Paper: ``Additional Smoothing Transition Autoregressive Models''
% Munich Personal RePEc Archive (MPRA) Paper No. 24657, 2010. 
%
% Nonlinear squares with no grid search and multi-step ahead forecasts.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nforecast = 4;
for jjj=1:nforecast
   % Load input data
%   load train_file.mat	% load the file with data

   y = train_data;
   g = mean(y);   
   t = length(y); % Tset up the delay 
   d = 0;
   x = lagmatrix(y,d); 
   
   % Set up the lag order
   i = 1;
   
   % Some initial values for c, gamma and be 
   c = 0;
   gamma = 1;
   model = 1;
   be    = 2;
   stdev = std(y);
   
   for p=1:i
     ylag(:,p) = lagmatrix(y,p);
   end;
   
   te   = length(ylag);
   ylag = ylag(i+d+1:t,:);
   y    = y(i+d+1:t,:);
   x    = x(i+d+1:t,:);   
   X1   = [ones(size(y)) ylag];
   
   if model==1	  % ESTAR (exponential)
     ESTAR_1 = 1-exp(-gamma.*((x-c).^2));
     Y       = X1;    
     for p=1:i+1
        h(:,p) = ESTAR_1.*Y(:,p);
     end;

     elseif model==2	% LSTAR (logistic)
       LSTAR_1 = 1./(1+exp(-gamma.*(x-c)));
       Y       = X1;    
       for p=1:i+1
         h(:,p) = LSTAR_1.*Y(:,p);
       end;

     elseif model==3  % TAHN_STAR (tangent hyperbolic)
       TSTAR_1 = (2./(1+exp(-2*gamma.*(x-c)))-1);
       Y       = X1;   
       for p=1:i+1
         h(:,p) = TSTAR_l.*Y(:,p);
       end;

     elseif model==4	% Generalized Bell function
       GBELL_STAR_1 = (1./(1+abs((x-c)/gamma).^2*be));
       Y            = X1;
       for p=1:i+1
         h(:, p) = GBELL_STAR_1.*Y(:,p);
       end;

     elseif model==5	% Gaussian
       GAUSS_STAR_1 = exp(-(x-c).^2/(2*gamma^2));
       Y            = X1;
       for p=1:i+1
         h(:,p) = GAUSS_STAR_1.*Y(:,p);
       end;
  end; % end if
  
  X  = [X1 h];
  X2 = [X1 x];

% Estimate if you want with OLS 
  bols = inv(X'*X)*X'*y;

  if model==1
    bols1          = [bols(1);bols(2);bols(3);bols(4);1;g];
    [sol,r,J,COVB] = nlinfit(X2,y,@ESTAR,bols1); 
    c1             = sol(end,:);
    gamma1         = sol(end-1,:);
    bols2          = sol(1:end-2,:);
 
  elseif model==2
    bols1          = [bols(1);bols(2);bols(3);bols(4);1;g];
    [sol,r,J,COVB] = nlinfit(X2,y,@LSTAR,bols1); 
    c1             = sol(end,:);
    gamma1         = sol(end-1,:);
    bols2          = sol(1:end-2,:);

  elseif model==3
    bols1          = [bols(1);bols(2);bols(3);bols(4);1;g];
    [sol,r,J,COVB] = nlinfit(X2,y,@TSTAR,bols1);
    c1             = sol(end,:);
    gamma1         = sol(end-1,:);
    bols2          = sol(l:end-2,:);

  elseif model==4
    bols1          = [bols(1);bols(2);bols(3);bols(4);g;1;2];
    [sol,r,J,COVB] = nlinfit(X2,y,@GBELL_STAR,bols1); 
    be1            = sol(end,:);
    gamma1         = sol(end-1,:);
    c1             = sol(end-2,:);
    bols2          = sol(1:end-3,:);

  elseif model==5
    bols1          = [bols(1);bols(2);bols(3);bols(4);1;g];
    [sol,r,J,COVB] = nlinfit(X2,y,@GAUSS_STAR,bols1); 
    c1             = sol(end,:);
    gamma1         = sol(end-1,:);
    bols2          = sol(1:end-2,:);
  end;  % end if

  if model==1	% ESTAR (exponential)
    ESTAR_1 = 1-exp(-gamma1.*((x-c1).^2));
    Y       = X1;
    f       = y(end,:)*ESTAR_1(end,:);
    for p=1:i+1
       h(:,p) = ESTAR_1.*Y(:,p);
    end;

  elseif model==2	% LSTAR (logistic)
    LSTAR_1 = 1./(1+exp(-gamma1.*(x-c1)));
    Y       = X1;
    f       = y(end,:)*LSTAR_1(end,:);
    for p=1:i+1
      h(:,p) = LSTAR_1.*Y(:,p);
    end;
  
  elseif model==3	% TAHN_STAR (tangent hyperbolic)
    TSTAR_1 = (2./(1+exp(-2*gamma1.*(x-c1)))-1);
    Y       = X1;
    f       = y(end,:)*TSTAR_1(end,:);
    for p=1:i+1
      h(:,p) = TSTAR_1.*Y(:,p);
    end;
  
  elseif model==4	% Generalized Bell function
    GBELL_STAR_1 = (1./(1+abs((x-c1)/gamma1).^2*be));
    Y            = X1;
    f            = y(end,:)*GBELL_STAR_1(end,:);
    for p=1:i+1
      h(:,p) = GBELL_STAR_1.*Y(:,p);
    end;
  
  elseif model==5
    GAUSS_STAR_l = exp(-(x-cl).^2/(2*gamma1^2));
    Y            = X1;
    f            = y(end,:)*GAUSS_STAR_1(end,:);
    for p=1:i+1
      h(:,p) = GAUSS_STAR_1.*Y(:,p);
    end; 
  end;  % end if
  
  X         = [Y h];
  yhat      = [X(end, 1) y(end,:) X(end,3) f]*bols2;
%%  data1   = [data1;yhat];
%%  save('can', 'data1');
end;  % end jjj=1:nforecast

X3        = [X1 h];
predict   = X3*bols2;
se        = sqrt(diag(COVB));	% Get coefficient standard errors
tstudent  = sol./se;
nforecast = 4;

%% load test_file.mat
test_y    = test_data(end-nforecast+1:end,:);
yfore     = train_data(end-nforecast+1:end,:);

figure,
plot(y,'-r'); 
hold on; 
plot(predict,'-b');
xlabel('Periods')
ylabel('Values')

title('In sample forecasts')
h1 = legend('Actual','forecasts');
figure, 
plot(test_y,'-r');
hold on; 
plot(yfore,'-b'); 
xlabel('Periods') 
ylabel('Values')

title('Out of sample forecasts') 
h = legend('Actual','forecasts');

