%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MATLAB code (miscellanea)
% Coded by: Eleftherios Giovanis 
% File: Matl_paper2_routine3.m
%
% Paper: ``Additional Smoothing Transition Autoregressive Models''
% Munich Personal RePEc Archive (MPRA) Paper No. 24657, 2010. 
%
% File: Matl-paper2-routine3.m
% 
% Nonlinear squares with grid search and one-step ahead forecasts.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  load file.mat	% load the file with data
y = data;
g = mean(y);

t = length(y); 
% Tset up the delay 
d = 1;
x = lagmatrix(y,d);

% Set up the lag order
i = 1;
% Some initial values for c, gamma and be
c     = 0;
gamma = 1;
model = 4;
be    = 2;
stdev = std(y);

for p=1:i
  ylag(:,p) = lagmatrix(y,p);
end;

ylag = ylag(i+d+1:t,:);
y    = y(i+d+1:t,:);
x    = x(i+d+1:t,:);
X1   = [ones(size(y)) ylag];

if model==1	% ESTAR (exponential)
  ESTAR_1 = 1-exp(-gamma.*((x-c).^2));
  Y       = X1;
  for p=1:i+1
    h(:,p) = ESTAR_1.*Y(:,p);
  end;
  
  elseif model==2	% LSTAR (logistic)
    LSTAR_1 = 1./(1+exp(-gamma.*(x-c)));
    Y       = X1;
    for p=1:i+1
      h(:,p) = LSTAR_1.*Y(:,p);
    end;
   
   elseif model==3	% TAHN_STAR (tangent hyperbolic)
   TSTAR_1 = (2./(1+exp(-2*gamma.*(x-c)))-1);
   Y       = X1;
   for p=1:i+1
     h(:,p) = TSTAR_1.*Y(:,p);
   end;
   
   elseif model==4     % Generalized Bell function
     GBELL_STAR_1 = (1./(1+abs((x-c)/gamma).^2*be));
     Y            = X1;
     for p=1:i+1
       h(:,p) = GBELL_STAR_1.*Y(:,p);
     end;
     
  elseif model==5	% Gaussian
    GAUSS_STAR_l = exp(-(x-c).^2/(2*gamma^2));
    Y            = X1;
    for p=1:i+1
      h(:,p) = GAUSS_STAR_1.*Y(:,p);
    end;
  
end; % end if

X  = [X1 h];
X2 = [X1 x];

% Estimate if you want with OLS to take initial values for estimated 
% coefficients 
bols = inv(X'*X)*X'*y;

grid_gamma = 1:0.5:3; 
grid_c     = -0.03:0.01:0.03;
grid_b     = 0.5:0.1:1;
gamma_f    = zeros(size(grid_gamma)); 
c_f        = zeros(size(grid_c));

if model==1
  for i = 1:length(grid_gamma)
     for j = 1:length(grid_c) 
       bols1            = [bols(1);bols(2);bols(3);bols(4);grid_gamma(i);grid_c(j)];
       [sol,res,J,COVB] = nlinfit(X2,y,@ESTAR,bols1);
       error(i,j)       = (y-X*sol(1:end-2,:))'*(y-X*sol(1:end-2,:));
     end;
  end;

elseif model==2 
  for i = 1:length(grid_gamma) 
    for j = 1:length(grid_c) 
      bols1            = [bols(1);bols(2);bols(3);bols(4);grid_gamma(i);grid_c(j)];
      [sol,res,J,COVB] = nlinfit(X2,y,@LSTAR,bols1);
      error(i,j)       = (y-X*sol(1:end-2,:))'*(y-X*sol(1:end-2,:));
    end;
  end;

elseif model==3
  for i = 1:length(grid_gamma)
    for j = 1:length(grid_c) 
       bols1            = [bols(1);bols(2);bols(3);bols(4);grid_gamma(i);grid_c(j)];
       [sol,res,J,COVB] = nlinfit(X2,y,@TSTAR,bols1); 
       error(i,j)       = (y-X*sol(1:end-2,:))'*(y-X*sol(1:end-2,:));
    end;
  end;

elseif model==4
  for i = 1:length(grid_gamma) 
     for j = 1:length(grid_c)
       for r = 1:length(grid_b)
         bols1              = [bols(1);bols(2);bols(3);bols(4);grid_c(j);grid_gamma(i);grid_b(r)];
         [sol,res,J,COVB]   = nlinfit(X2,y,@GBELL_STAR,bols1);
         error.field(i,j,r) = (y-X*sol(1 :end-3,:))'*(y-X*sol(1:end-3,:)); 
       end;
     end;
  end;

elseif model==5
  for i = 1:length(grid_gamma) 
     for j = 1:length(grid_c) 
        bols1            = [bols(1);bols(2);bols(3);bols(4);grid_c(j);grid_gamma(i)];
        [sol,res,J,COVB] = nlinfit(X2,y,@GAUSS_STAR,bols1);
        error(i,j)       = (y-X*sol(1:end-2,:))'*(y-X*sol(1:end-2,:));
     end;
  end;

end;  % end if

nforecast = 20;
%% load file.mat 
bols2 = sol(1:end-2,:); 

for ii=nforecast:-1:1 
  y     = data(end-100:end-ii,1);  
  d     = 1;
  x     = lagmatrix(y,d);
  i     = 1; 
  c     = 0.01;
  gamma = 1.5;
  model = 1;
  be    = 2;
  
  clear ylag 
  for p=1:i
     ylag(:,p) = lagmatrix(y,p);
  end;

  te   = length(ylag);
  t    = length(y);
  ylag = ylag(i+d+1:t,:);
  y    = y(i+d+1:t,:);
  x    = x(i+d+1:t,:);
  X1   = [ones(size(y)) ylag];

  if model==1	% ESTAR (exponential)
    ESTAR_1 = 1-exp(-gamma.*((x-c).^2));
    Y       = X1;
    f       = y(end,:)*ESTAR_1(end,:);
    clear h 
    for p=1:i+1
      h(:,p) = ESTAR_1.*Y(:,p);
    end;
  
  elseif model==2	% LSTAR (logistic)
    LSTAR_1 = 1./(1+exp(-gamma.*(x-c)));
    Y       = X1;
    f       = y(end,:)*LSTAR_1(end,:);
    clear h 
    for p=1:i+1
      h(:,p) = LSTAR_1.*Y(:,p);
    end;
    
    elseif model==3	% TAHN_STAR (tangent hyperbolic)
      TSTAR_1 = (2./(1+exp(-2*gamma.*(x-c)))-1);
      Y       = X1;
      f       = y(end,:)*TSTAR_1(end,:);
      clear h 
      for p=1:i+1
        h(:,p) = TSTAR_1.*Y(:,p);
      end;
      
    elseif model==4	% Generalized Bell function
      GBELL_STAR_1 = (1./(1+abs((x-c)/gamma).^2*be));
      Y            = X1;
      f            = y(end,:)*GBELL_STAR_1(end,:);
      clear h 
      for p=1:i+1
         h(:,p) = GBELL_STAR_1.*Y(:,p);
      end;
      
   elseif model==5	% Gaussian
     GAUSS_STAR_1 = exp(-(x - c).^2/(2*gamma^2));
     Y            = X1;
     f            = y(end,:)*GAUSS_STAR_1(end,:);
     clear h
     for p=1:i+1
       h(:,p) = GAUSS_STAR_1.*Y(:,p);
     end;
  
   end; % end if

   X          = [Y h];
   yhat(ii,:) = [X(end,1) y(end,:) X(end,3) f]*bols2;

end;   % end loop ii

for iii=1:nforecast
  yfore(iii,:) = yhat(end-iii+1,:);
end;
